import time

from Acquisition import aq_base, aq_inner
from plone.i18n.normalizer import FILENAME_REGEX
from plone.i18n.normalizer.interfaces import IURLNormalizer, IUserPreferredURLNormalizer
from zope.component import getUtility
from zope.container.interfaces import INameChooser
from zope.interface import implementer

from plone.app.content.interfaces import INameFromTitle

ATTEMPTS = 100


@implementer(INameChooser)
class NormalizingNameChooser:
    """A name chooser for a Zope object manager.

    If the object is adaptable to or provides INameFromTitle, use the
    title to generate a name.
    """

    def __init__(self, context):
        self.context = context

    def checkName(self, name, obj):
        return not self._getCheckId(obj)(name, required=1)

    def chooseName(self, name, obj):
        container = aq_inner(self.context)
        if not name:
            nameFromTitle = INameFromTitle(obj, None)
            if nameFromTitle is not None:
                name = nameFromTitle.title
            if not name:
                name = getattr(aq_base(obj), "id", None)
            if not name:
                name = getattr(aq_base(obj), "portal_type", None)
            if not name:
                name = obj.__class__.__name__

        if not isinstance(name, str):
            name = str(name, "utf-8")
            # name = name.encode('utf-8')

        request = getattr(obj.__of__(container), "REQUEST", None)
        if request is not None:
            name = IUserPreferredURLNormalizer(request).normalize(name)
        else:
            name = getUtility(IURLNormalizer).normalize(name)

        return self._findUniqueName(name, obj)

    def _findUniqueName(self, name, obj):
        """Find a unique name in the parent folder, based on the given id, by
        appending -n, where n is a number greater than 1, or just the id if
        it's ok.
        """
        check_id = self._getCheckId(obj)

        if not check_id(name, required=1):
            return name

        ext = ""
        m = FILENAME_REGEX.match(name)
        if m is not None:
            name = m.groups()[0]
            ext = "." + m.groups()[1]

        idx = 1
        while idx <= ATTEMPTS:
            new_name = "%s-%d%s" % (name, idx, ext)
            if not check_id(new_name, required=1):
                return new_name
            idx += 1

        # give one last attempt using the current date time before giving up
        new_name = f"{name}-{time.time()}{ext}"
        if not check_id(new_name, required=1):
            return new_name

        raise ValueError(
            "Cannot find a unique name based on %s after %d attemps."
            % (
                name,
                ATTEMPTS,
            )
        )

    def _getCheckId(self, obj):
        """Return a function that can act as the check_id script."""
        parent = aq_inner(self.context)
        # Check for a method or a skin script, like
        # Products/CMFPlone/skins/plone_scripts/check_id.py until Plone 5.1.
        _check_id = getattr(obj, "check_id", None)

        def do_Plone_check(newid, required):
            if _check_id is not None:
                return _check_id(newid, required=required, contained_by=parent)

            from Products.CMFPlone.utils import check_id

            return check_id(obj, newid, required=required, contained_by=parent)

        return do_Plone_check



import unittest

import transaction
from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, logout, setRoles
from plone.locking.interfaces import ILockable
from plone.testing.zope import Browser
from z3c.form.interfaces import IFormLayer
from zExceptions import Unauthorized
from zope.component import getMultiAdapter
from zope.interface import alsoProvides

from plone.app.content.testing import PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING


class ActionsDXTestCase(unittest.TestCase):

    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        self.portal.acl_users.userFolderAddUser("editor", "secret", ["Editor"], [])

        # For z3c.forms request must provide IFormLayer
        alsoProvides(self.request, IFormLayer)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        self.portal.invokeFactory(type_name="Folder", id="f1", title="A Tést Folder")

        transaction.commit()
        self.browser = Browser(self.layer["app"])
        self.browser.handleErrors = False
        self.browser.addHeader(
            "Authorization", "Basic {}:{}".format(TEST_USER_NAME, "secret")
        )

    def tearDown(self):
        if "f1" in self.portal.objectIds():
            self.portal.manage_delObjects(ids="f1")
            transaction.commit()

    def test_delete_confirmation(self):
        folder = self.portal["f1"]

        form = getMultiAdapter((folder, self.request), name="delete_confirmation")
        form.update()

        cancel = form.buttons["Cancel"]
        form.handlers.getHandler(cancel)(form, form)

        self.assertFalse(form.is_locked)

    def test_delete_confirmation_if_locked(self):
        folder = self.portal["f1"]
        lockable = ILockable.providedBy(folder)

        form = getMultiAdapter((folder, self.request), name="delete_confirmation")
        form.update()

        self.assertFalse(form.is_locked)

        if lockable:
            lockable.lock()

        form = getMultiAdapter((folder, self.request), name="delete_confirmation")
        form.update()

        self.assertFalse(form.is_locked)

        # After switching the user it should not be possible to delete the
        # object. Of course this is only possible if our context provides
        # ILockable interface.
        if lockable:
            logout()
            login(self.portal, "editor")

            form = getMultiAdapter((folder, self.request), name="delete_confirmation")
            form.update()
            self.assertTrue(form.is_locked)

            logout()
            login(self.portal, TEST_USER_NAME)

            ILockable(folder).unlock()

    def test_delete_confirmation_cancel(self):
        folder = self.portal["f1"]

        self.browser.open(folder.absolute_url() + "/delete_confirmation")
        self.browser.getControl(name="form.buttons.Cancel").click()
        context_state = getMultiAdapter(
            (folder, self.request), name="plone_context_state"
        )
        self.assertEqual(self.browser.url, context_state.view_url())

    def prepare_for_acquisition_tests(self):
        """create content and an alternate authenticated browser session

        creates the following content structure:

        |-- f1
        |   |-- test
        |-- test
        """
        # create a page at the root and one nested with the same id.
        p1 = self.portal.invokeFactory(
            type_name="Document", id="test", title="Test Page at Root"
        )
        folder_1 = self.portal["f1"]
        p2 = folder_1.invokeFactory(
            type_name="Document", id="test", title="Test Page in Folder"
        )
        contained_test_page = folder_1[p2]

        transaction.commit()

        # create an alternate browser also logged in with manager
        browser_2 = Browser(self.layer["app"])
        browser_2.handleErrors = False
        browser_2.addHeader(
            "Authorization", "Basic {}:{}".format(TEST_USER_NAME, "secret")
        )

        # return the id of the root page, the nested page itself, and the
        # alternate browser
        return p1, contained_test_page, browser_2

    def test_delete_wrong_object_by_acquisition_with_action(self):
        """exposes delete-by-acquisition bug using the delete action

        see https://github.com/plone/Products.CMFPlone/issues/383
        """
        p1_id, page_2, browser_2 = self.prepare_for_acquisition_tests()

        # open two different browsers to the 'delete confirmation' view
        delete_url = page_2.absolute_url() + "/delete_confirmation"
        self.browser.open(delete_url)
        browser_2.open(delete_url)
        self.assertTrue(p1_id in self.portal)
        for browser in [self.browser, browser_2]:
            browser.getControl(name="form.buttons.Delete").click()

        # the nested folder should be gone, but the one at the root should
        # remain.
        self.assertFalse(page_2.id in self.portal["f1"])
        self.assertTrue(p1_id in self.portal)

    def test_rename_form(self):
        logout()
        folder = self.portal["f1"]

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(folder.absolute_url() + "/folder_rename")
        self.browser.getControl(name="form.widgets.new_id").value = "f2"
        self.browser.getControl(name="form.widgets.new_title").value = "F2"
        self.browser.getControl(name="form.buttons.Rename").click()
        self.assertEqual(folder.getId(), "f2")
        self.assertEqual(folder.Title(), "F2")
        self.assertEqual(self.browser.url, folder.absolute_url())

        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(ids="f2")
        transaction.commit()

    def test_rename_form_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry["plone.types_use_view_action_in_listings"] = [
            "Image",
            "File",
            "Document",
        ]

        folder = self.portal["f1"]
        folder.invokeFactory("Document", "document1")
        document1 = folder["document1"]
        transaction.commit()
        logout()

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(document1.absolute_url() + "/object_rename")
        self.browser.getControl(name="form.widgets.new_id").value = "f2"
        self.browser.getControl(name="form.widgets.new_title").value = "F2"
        self.browser.getControl(name="form.buttons.Rename").click()
        self.assertEqual(document1.getId(), "f2")
        self.assertEqual(document1.Title(), "F2")
        self.assertEqual(self.browser.url, document1.absolute_url() + "/view")

        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(ids="f1")
        transaction.commit()

    def test_create_safe_id_on_renaming(self):
        logout()
        folder = self.portal["f1"]

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(folder.absolute_url() + "/folder_rename")
        self.browser.getControl(name="form.widgets.new_id").value = " ? f4 4 "
        self.browser.getControl(name="form.widgets.new_title").value = " F2 "
        self.browser.getControl(name="form.buttons.Rename").click()
        self.assertEqual(folder.getId(), "f4-4")
        self.assertEqual(folder.Title(), "F2")
        self.assertEqual(self.browser.url, folder.absolute_url())

        login(self.portal, TEST_USER_NAME)
        self.portal.manage_delObjects(ids="f4-4")
        transaction.commit()

    def test_default_page_updated_on_rename(self):
        login(self.portal, TEST_USER_NAME)
        folder = self.portal["f1"]
        folder.invokeFactory(type_name="Document", id="d1", title="A Doc")
        doc = folder["d1"]
        folder.setDefaultPage("d1")
        transaction.commit()
        self.assertEqual(folder.getDefaultPage(), "d1")

        # We need zope2.CopyOrMove permission to rename content
        self.browser.open(doc.absolute_url() + "/object_rename")
        self.browser.getControl(name="form.widgets.new_id").value = " ?renamed"
        self.browser.getControl(name="form.widgets.new_title").value = "Doc"
        self.browser.getControl(name="form.buttons.Rename").click()
        self.assertEqual(folder.getFolderContents()[0].id, "renamed")
        self.assertEqual(folder.getDefaultPage(), "renamed")

    def test_rename_form_cancel(self):
        folder = self.portal["f1"]

        _id = folder.getId()
        _title = folder.Title()

        self.browser.open(folder.absolute_url() + "/folder_rename")
        self.browser.getControl(name="form.buttons.Cancel").click()
        transaction.commit()

        self.assertEqual(self.browser.url, folder.absolute_url())
        self.assertEqual(folder.getId(), _id)
        self.assertEqual(folder.Title(), _title)

    def test_rename_form_cancel_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry["plone.types_use_view_action_in_listings"] = [
            "Image",
            "File",
            "Document",
        ]
        folder = self.portal["f1"]
        folder.invokeFactory("Document", "document1")
        document1 = folder["document1"]
        transaction.commit()

        _id = document1.getId()
        _title = document1.Title()

        self.browser.open(document1.absolute_url() + "/object_rename")
        self.browser.getControl(name="form.buttons.Cancel").click()
        transaction.commit()

        self.assertEqual(self.browser.url, document1.absolute_url() + "/view")
        self.assertEqual(document1.getId(), _id)
        self.assertEqual(document1.Title(), _title)

    def _get_token(self, context):
        authenticator = getMultiAdapter((context, self.request), name="authenticator")

        return authenticator.token()

    def test_object_cut_view(self):
        folder = self.portal["f1"]

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized, self.browser.open, f"{folder.absolute_url():s}/object_cut"
        )

        # We need to have Copy or Move permission to cut an object
        self.browser.open(
            "{:s}/object_cut?_authenticator={:s}".format(
                folder.absolute_url(), self._get_token(folder)
            )
        )

        self.assertIn("__cp", self.browser.cookies)
        self.assertIn(f"{folder.Title():s} cut.", self.browser.contents)

    def test_object_cut_view_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry["plone.types_use_view_action_in_listings"] = [
            "Image",
            "File",
            "Document",
        ]
        folder = self.portal["f1"]
        folder.invokeFactory("Document", "document1")
        document1 = folder["document1"]
        transaction.commit()

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized, self.browser.open, f"{document1.absolute_url():s}/object_cut"
        )

        # We need to have Copy or Move permission to cut an object
        self.browser.open(
            "{:s}/object_cut?_authenticator={:s}".format(
                document1.absolute_url(), self._get_token(document1)
            )
        )

        self.assertIn("__cp", self.browser.cookies)
        self.assertIn(f"{document1.Title():s} cut.", self.browser.contents)
        self.assertEqual(document1.absolute_url() + "/view", self.browser.url)

    def test_object_copy_view(self):
        folder = self.portal["f1"]

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized, self.browser.open, f"{folder.absolute_url():s}/object_copy"
        )

        self.browser.open(
            "{:s}/object_copy?_authenticator={:s}".format(
                folder.absolute_url(), self._get_token(folder)
            )
        )

        self.assertIn("__cp", self.browser.cookies)
        self.assertIn(f"{folder.Title():s} copied.", self.browser.contents)

    def test_object_copy_with_view_action(self):
        # can't be bothered to register blobs, instead we add documents to
        # typesUseViewActionInListings
        registry = self.portal.portal_registry
        registry["plone.types_use_view_action_in_listings"] = [
            "Image",
            "File",
            "Document",
        ]

        folder = self.portal["f1"]
        folder.invokeFactory("Document", "document1")
        document1 = folder["document1"]
        transaction.commit()

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized, self.browser.open, f"{document1.absolute_url():s}/object_copy"
        )

        self.browser.open(
            "{:s}/object_copy?_authenticator={:s}".format(
                document1.absolute_url(), self._get_token(document1)
            )
        )

        self.assertIn("__cp", self.browser.cookies)
        self.assertIn(f"{document1.Title():s} copied.", self.browser.contents)
        self.assertEqual(document1.absolute_url() + "/view", self.browser.url)

    def test_object_cut_and_paste(self):
        folder = self.portal["f1"]
        self.portal.invokeFactory(type_name="Document", id="d1", title="A Doc")
        doc = self.portal["d1"]
        transaction.commit()

        self.browser.open(
            "{:s}/object_cut?_authenticator={:s}".format(
                doc.absolute_url(), self._get_token(doc)
            )
        )

        self.assertIn("__cp", self.browser.cookies)
        self.assertIn("d1", self.portal.objectIds())
        self.assertIn("f1", self.portal.objectIds())

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized, self.browser.open, f"{folder.absolute_url():s}/object_paste"
        )

        self.browser.open(
            "{:s}/object_paste?_authenticator={:s}".format(
                folder.absolute_url(), self._get_token(doc)
            )
        )

        self.assertIn("__cp", self.browser.cookies)
        transaction.commit()

        self.assertNotIn("d1", self.portal.objectIds())
        self.assertIn("d1", folder.objectIds())
        self.assertIn("Item(s) pasted.", self.browser.contents)

    def test_object_copy_and_paste(self):
        folder = self.portal["f1"]
        folder.invokeFactory(type_name="Document", id="d1", title="A Doc")
        doc = folder["d1"]
        transaction.commit()

        self.browser.open(
            "{:s}/object_copy?_authenticator={:s}".format(
                doc.absolute_url(), self._get_token(doc)
            )
        )

        self.assertIn("__cp", self.browser.cookies)

        # We need pass an authenticator token to prevent Unauthorized
        self.assertRaises(
            Unauthorized, self.browser.open, f"{folder.absolute_url():s}/object_paste"
        )

        self.browser.open(
            "{:s}/object_paste?_authenticator={:s}".format(
                folder.absolute_url(), self._get_token(folder)
            )
        )
        transaction.commit()

        self.assertIn("f1", self.portal.objectIds())
        self.assertIn("d1", folder.objectIds())
        self.assertIn("copy_of_d1", folder.objectIds())
        self.assertIn("Item(s) pasted.", self.browser.contents)

    def test_object_copy_and_paste_multiple_times(self):
        folder = self.portal["f1"]
        folder.invokeFactory(type_name="Document", id="d1", title="A Doc")
        doc = folder["d1"]
        transaction.commit()

        self.browser.open(
            "{:s}/object_copy?_authenticator={:s}".format(
                doc.absolute_url(), self._get_token(doc)
            )
        )

        self.assertIn("__cp", self.browser.cookies)
        self.browser.open(
            "{:s}/object_paste?_authenticator={:s}".format(
                folder.absolute_url(), self._get_token(folder)
            )
        )
        self.browser.open(
            "{:s}/object_paste?_authenticator={:s}".format(
                folder.absolute_url(), self._get_token(folder)
            )
        )

        # Cookie should persist, because you can paste the item multiple times
        self.assertIn("__cp", self.browser.cookies)
        self.assertIn("f1", self.portal.objectIds())
        self.assertIn("d1", folder.objectIds())
        self.assertIn("copy_of_d1", folder.objectIds())
        self.assertIn("copy2_of_d1", folder.objectIds())
        self.assertIn("Item(s) pasted.", self.browser.contents)



import doctest
import unittest

from zope.component.testing import setUp, tearDown


def test_suite():
    return unittest.TestSuite(
        doctest.DocFileSuite(
            "namechooser.txt",
            package="plone.app.content",
            optionflags=doctest.ELLIPSIS,
            setUp=setUp,
            tearDown=tearDown,
        )
    )


if __name__ == "__main__":
    unittest.main(defaultTest="test_suite")



from urllib.parse import quote_plus

from Acquisition import aq_inner, aq_parent
from plone.i18n.normalizer.interfaces import IIDNormalizer
from plone.memoize.instance import memoize
from plone.memoize.request import memoize_diy_request
from plone.protect.authenticator import createToken
from Products.CMFCore.Expression import createExprContext
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone.interfaces.constrains import ISelectableConstrainTypes
from zope.component import getMultiAdapter, queryUtility
from zope.i18n import translate
from zope.publisher.browser import BrowserView

from plone.app.content.browser.interfaces import IFolderContentsView


@memoize_diy_request(arg=0)
def _allowedTypes(request, context):
    return context.allowedContentTypes()


class FolderFactoriesView(BrowserView):
    """The folder_factories view - show addable types"""

    def __call__(self):
        if "form.button.Add" in self.request.form:
            urltool = getToolByName(self.context, "portal_url")
            url = self.request.form.get("url")
            if not urltool.isURLInPortal(url):
                url = self.context.absolute_url()
            self.request.response.redirect(url)
            return ""
        else:
            return self.index()

    def can_constrain_types(self):
        aspect = ISelectableConstrainTypes(self.add_context(), None)
        return aspect.canSetConstrainTypes() if aspect else False

    @memoize
    def add_context(self):
        context = self.context
        context_state = getMultiAdapter(
            (context, self.request), name="plone_context_state"
        )
        context = aq_inner(context)
        try:
            published = self.request.PUBLISHED
        except AttributeError:
            published = context
        if context_state.is_structural_folder():
            if context_state.is_default_page():
                is_folder_contents_view = IFolderContentsView.providedBy(published)
                if is_folder_contents_view or self == published:
                    # on the folder_contents view and factories view,
                    # show the actual context object's addable types
                    return context
                else:
                    return aq_parent(context)
            else:
                return context
        else:
            return aq_parent(context)

    # NOTE: This is also used by plone.app.contentmenu.menu.FactoriesMenu.
    # The return value is somewhat dictated by the menu infrastructure, so
    # be careful if you change it

    def addable_types(self, include=None):
        """Return menu item entries in a TAL-friendly form.

        Pass a list of type ids to 'include' to explicitly allow a list of
        types.
        """

        context = aq_inner(self.context)
        request = self.request

        results = []

        idnormalizer = queryUtility(IIDNormalizer)
        portal_state = getMultiAdapter((context, request), name="plone_portal_state")

        addContext = self.add_context()
        baseUrl = addContext.absolute_url()
        token = createToken()

        allowedTypes = _allowedTypes(request, addContext)

        types_tool = getToolByName(context, "portal_types")

        # Note: we don't check 'allowed' or 'available' here, because these are
        # slow. We assume the 'allowedTypes' list has already performed the
        # necessary calculations
        actions = types_tool.listActionInfos(
            object=addContext,
            check_permissions=False,
            check_condition=False,
            category="folder/add",
        )
        addActionsById = {a["id"]: a for a in actions}

        expr_context = createExprContext(
            aq_parent(addContext), portal_state.portal(), addContext
        )
        for t in allowedTypes:
            typeId = t.getId()
            if include is None or typeId in include:
                cssId = idnormalizer.normalize(typeId)
                cssClass = "contenttype-%s" % cssId

                url = None
                addAction = addActionsById.get(typeId, None)
                if addAction is not None:
                    url = addAction["url"]

                if not url:
                    url = "{}/createObject?type_name={}&_authenticator={}".format(
                        baseUrl, quote_plus(typeId), token
                    )

                icon = t.getIconExprObject()
                if icon:
                    icon = icon(expr_context)

                results.append(
                    {
                        "id": typeId,
                        "title": t.Title(),
                        "description": t.Description(),
                        "action": url,
                        "selected": False,
                        "icon": icon,
                        "extra": {"id": cssId, "separator": None, "class": cssClass},
                        "submenu": None,
                    }
                )

        # Sort the addable content types based on their translated title
        results = [
            (translate(ctype["title"], context=request), ctype) for ctype in results
        ]
        results.sort()
        results = [ctype[-1] for ctype in results]

        return results



import doctest

from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import (
    PLONE_FIXTURE,
    FunctionalTesting,
    IntegrationTesting,
    PloneSandboxLayer,
    applyProfile,
)
from Products.CMFCore.utils import getToolByName
from zope.configuration import xmlconfig
from zope.interface import implementer, provider
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm, SimpleVocabulary


@implementer(IVocabularyFactory)
class ExampleVocabulary:
    def __call__(self, context, query=None):
        items = ["One", "Two", "Three"]
        tmp = SimpleVocabulary(
            [
                SimpleTerm(it.lower(), it.lower(), it)
                for it in items
                if query is None or query.lower() in it.lower()
            ]
        )
        tmp.test = 1
        return tmp


@provider(IVocabularyFactory)
def ExampleFunctionVocabulary(context, query=None):
    items = ["First", "Second", "Third"]
    tmp = SimpleVocabulary(
        [
            SimpleTerm(it.lower(), it.lower(), it)
            for it in items
            if query is None or query.lower() in it.lower()
        ]
    )
    return tmp


class PloneAppContent(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    USER_NAME = "johndoe"
    USER_PASSWORD = "secret"
    MEMBER_NAME = "janedoe"
    MEMBER_PASSWORD = "secret"
    USER_WITH_FULLNAME_NAME = "jim"
    USER_WITH_FULLNAME_FULLNAME = "Jim Fulton"
    USER_WITH_FULLNAME_PASSWORD = "secret"
    MANAGER_USER_NAME = "manager"
    MANAGER_USER_PASSWORD = "secret"

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.app.content

        xmlconfig.file(
            "configure.zcml", plone.app.content, context=configurationContext
        )

    def setUpPloneSite(self, portal):
        # Creates some users
        acl_users = getToolByName(portal, "acl_users")
        acl_users.userFolderAddUser(
            self.USER_NAME,
            self.USER_PASSWORD,
            [],
            [],
        )
        acl_users.userFolderAddUser(
            self.MEMBER_NAME,
            self.MEMBER_PASSWORD,
            ["Member"],
            [],
        )
        acl_users.userFolderAddUser(
            self.USER_WITH_FULLNAME_NAME,
            self.USER_WITH_FULLNAME_PASSWORD,
            ["Member"],
            [],
        )
        mtool = getToolByName(portal, "portal_membership", None)
        mtool.addMember("jim", "Jim", ["Member"], [])
        mtool.getMemberById("jim").setMemberProperties(
            {"fullname": "Jim Fult\xc3\xb8rn"}
        )

        acl_users.userFolderAddUser(
            self.MANAGER_USER_NAME,
            self.MANAGER_USER_PASSWORD,
            ["Manager"],
            [],
        )
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")


class NonAsciiLayer(PloneSandboxLayer):
    def setUpZope(self, app, configurationContext):
        import plone.app.content.tests

        xmlconfig.file(
            "profiles/non-ascii-workflow.zcml",
            plone.app.content.tests,
            context=configurationContext,
        )

    def setUpPloneSite(self, portal):
        # applyProfile which has non-ascii characters in state titles
        applyProfile(portal, "plone.app.content.tests:non-ascii-workflow")


PLONE_APP_CONTENT_FIXTURE = PloneAppContent()
PLONE_APP_CONTENT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENT_FIXTURE,), name="PloneAppContent:Integration"
)
PLONE_APP_CONTENT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENT_FIXTURE,), name="PloneAppContent:Functional"
)


# Dexterity test layers
PLONE_APP_CONTENT_DX_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,), name="PloneAppContentDX:Integration"
)
PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,), name="PloneAppContentDX:Functional"
)


# Test layer with a workflow containing non-ascii characters in state titles.
PLONE_APP_CONTENT_NON_ASCII_LAYER = NonAsciiLayer()
PLONE_APP_CONTENT_NON_ASCII_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENT_NON_ASCII_LAYER,),
    name="PloneAppContentNonAscii:Integration",
)


optionflags = (
    doctest.REPORT_ONLY_FIRST_FAILURE | doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
)



from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import human_readable_size, isExpired
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from z3c.form import field, form
from zope.interface import Interface
from zope.publisher.browser import BrowserView
from zope.schema import Datetime
from zope.schema.fieldproperty import FieldProperty


class IContentStatusHistoryDates(Interface):
    """Interface for the two dates on content status history view"""

    effective_date = Datetime(
        title=_("label_effective_date", default="Publishing Date"),
        description=_(
            "help_effective_date",
            default="The date when the item will be published. If no "
            "date is selected the item will be published immediately.",
        ),
        required=False,
    )

    expiration_date = Datetime(
        title=_("label_expiration_date", default="Expiration Date"),
        description=_(
            "help_expiration_date",
            default="The date when the item expires. This will automatically "
            "make the item invisible for others at the given date. "
            "If no date is chosen, it will never expire.",
        ),
        required=False,
    )


class ContentStatusHistoryDatesForm(form.Form):
    fields = field.Fields(IContentStatusHistoryDates)
    ignoreContext = True
    label = "Content status history dates"

    effective_date = FieldProperty(IContentStatusHistoryDates["effective_date"])
    expiration_date = FieldProperty(IContentStatusHistoryDates["expiration_date"])


class ContentStatusHistoryView(BrowserView):

    template = ViewPageTemplateFile("templates/content_status_history.pt")

    def __init__(self, context, request):
        super().__init__(context, request)

        self.dates_form = ContentStatusHistoryDatesForm(context, request)
        self.dates_form.updateWidgets()
        self.plone_utils = getToolByName(context, "plone_utils")
        self.errors = {}

    def __call__(
        self,
        workflow_action=None,
        paths=[],
        comment="",
        effective_date=None,
        expiration_date=None,
        include_children=False,
        *args
    ):

        data = self.dates_form.extractData()
        if self.request.get("form.widgets.effective_date-calendar", None) and data:
            effective_date = data[0]["effective_date"].strftime("%Y-%m-%d %H:%M")

        if self.request.get("form.widgets.expiration_date-calendar", None) and data:
            expiration_date = data[0]["expiration_date"].strftime("%Y-%m-%d %H:%M")

        if self.request.get("form.button.Cancel", None):
            return self.request.RESPONSE.redirect(
                "%s/view" % self.context.absolute_url()
            )

        if self.request.get("form.submitted", None):
            self.validate(workflow_action=workflow_action, paths=paths)
            if self.errors:
                self.plone_utils.addPortalMessage(
                    _("Please correct the indicated errors."), "error"
                )
                return self.template()

        if self.request.get("form.button.Publish", None):
            return self.context.restrictedTraverse("content_status_modify")(
                workflow_action=workflow_action,
                comment=comment,
                effective_date=effective_date,
                expiration_date=expiration_date,
            )

        if self.request.get("form.button.FolderPublish", None):
            self.context.restrictedTraverse("folder_publish")(
                workflow_action=workflow_action,
                paths=paths,
                comment=comment,
                expiration_date=expiration_date,
                effective_date=effective_date,
                include_children=include_children,
            )

        return self.template()

    def validate(self, workflow_action=None, paths=[]):
        if workflow_action is None:
            self.errors["workflow_action"] = _("You must select a publishing action.")

        if not paths:
            self.errors["paths"] = _("You must select content to change.")
            # If there are no paths, it's mostly a mistake
            # Set paths using orgi_paths, otherwise users are getting confused
            orig_paths = self.request.get("orig_paths")
            self.request.set("paths", orig_paths)

    def get_objects_from_path_list(self, paths=[]):
        contents = []
        portal = getToolByName(self.context, "portal_url").getPortalObject()
        for path in paths:
            obj = portal.restrictedTraverse(str(path), None)
            if obj is not None:
                contents.append(obj)
        return contents

    def redirect_to_referrer(self):
        referer = self.request.get("HTTP_REFERER", "")
        target_url = referer.split("?", 1)[0]
        return self.request.RESPONSE.redirect(target_url)

    def isExpired(self, content):
        return isExpired(content)

    def human_readable_size(self, size):
        return human_readable_size(size)



from AccessControl import Unauthorized
from Acquisition import aq_inner
from plone.app.uuid.utils import uuidToCatalogBrain
from plone.protect.postonly import check as checkpost
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone import utils
from Products.CMFPlone.interfaces.controlpanel import ISiteSchema
from Products.CMFPlone.utils import get_top_site_from_url
from Products.Five import BrowserView
from Products.PortalTransforms.transforms.safe_html import SafeHTML
from zope.browsermenu.interfaces import IBrowserMenu
from zope.component import getMultiAdapter, getUtilitiesFor, getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory

from plone.app.content.browser.file import TUS_ENABLED
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.app.content.interfaces import IStructureAction
from plone.app.content.utils import json_dumps, json_loads


class ContentsBaseAction(BrowserView):

    success_msg = _("Success")
    failure_msg = _("Failure")
    required_obj_permission = None

    @property
    def site(self):
        return get_top_site_from_url(self.context, self.request)

    def objectTitle(self, obj):
        context = aq_inner(obj)
        title = utils.pretty_title_or_id(context, context)
        return utils.safe_unicode(title)

    def protect(self):
        authenticator = getMultiAdapter(
            (self.context, self.request), name="authenticator"
        )
        if not authenticator.verify():
            raise Unauthorized
        checkpost(self.request)

    def json(self, data):
        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )
        return json_dumps(data)

    def get_selection(self):
        selection = self.request.form.get("selection", "[]")
        return json_loads(selection)

    def action(self, obj):
        """
        fill in this method to do action against each item in the selection
        """
        pass

    def finish(self):
        pass

    def __call__(self, keep_selection_order=False):
        self.protect()
        self.errors = []
        context = aq_inner(self.context)
        selection = self.get_selection()

        parts = str(self.request.form.get("folder", "").lstrip("/")).split("/")
        if parts:
            parent = self.site.unrestrictedTraverse("/".join(parts[:-1]))
            self.dest = parent.restrictedTraverse(parts[-1])

        self.catalog = getToolByName(context, "portal_catalog")
        self.mtool = getToolByName(self.context, "portal_membership")

        brains = []
        if keep_selection_order:
            brains = [uuidToCatalogBrain(uid) for uid in selection]
        else:
            brains = self.catalog(UID=selection, show_inactive=True)

        for brain in brains:
            if not brain:
                continue
            # remove everyone so we know if we missed any
            selection.remove(brain.UID)
            obj = brain.getObject()
            if self.required_obj_permission and not self.mtool.checkPermission(
                self.required_obj_permission, obj
            ):
                self.errors.append(
                    _(
                        'Permission denied for "${title}"',
                        mapping={"title": self.objectTitle(obj)},
                    )
                )
                continue
            self.action(obj)

        self.finish()
        return self.message(selection)

    def message(self, missing=[]):
        if len(missing) > 0:
            self.errors.append(
                _("${items} could not be found", mapping={"items": str(len(missing))})
            )
        if self.errors:
            msg = self.failure_msg
        else:
            msg = self.success_msg

        translated_msg = translate(msg, context=self.request)
        if self.errors:
            translated_errors = [
                translate(error, context=self.request) for error in self.errors
            ]
            translated_msg = "{:s}: {:s}".format(
                translated_msg, "\n".join(translated_errors)
            )

        return self.json(
            {"status": "warning" if self.errors else "success", "msg": translated_msg}
        )


@implementer(IFolderContentsView)
class FolderContentsView(BrowserView):
    def get_actions(self):
        actions = []
        for name, Utility in getUtilitiesFor(IStructureAction):
            utility = Utility(self.context, self.request)
            actions.append(utility)
        actions.sort(key=lambda a: a.order)
        return [a.get_options() for a in actions]

    @property
    def ignored_columns(self):
        """Return columns, which should be ignored in folder contents."""
        # These columns either have alternatives or are probably not useful
        ignored = [
            "Date",
            "Title",
            "author_name",
            "cmf_uid",
            "commentators",
            "created",
            "effective",
            "expires",
            "getIcon",
            "getMimeIcon",
            "getId",
            "getRemoteUrl",
            "in_response_to",
            "listCreators",
            "meta_type",
            "modified",
            "portal_type",
            "sync_uid",
        ]
        return ignored

    def get_columns(self):
        columns = {}
        voc = getUtility(IVocabularyFactory, "plone.app.vocabularies.MetadataFields")(
            self.context
        )
        for term in voc:
            if term.value not in self.ignored_columns:
                columns[term.value] = translate(term.title, context=self.request)

        return columns

    def get_thumb_scale(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        if settings.no_thumbs_tables:
            # thumbs to be supressed
            return None
        thumb_scale_table = settings.thumb_scale_table
        return thumb_scale_table

    def default_page_types(self):
        registry = getUtility(IRegistry)
        return registry.get("plone.default_page_types", [])

    @property
    def ignored_indexes(self):
        ignored = [
            "Date",
            "Description",
            "Title",
            "allowedRolesAndUsers",
            "author_name",
            "cmf_uid",
            "commentators",
            "effectiveRange",
            "getId",
            "getObjectPositionInParent",
            "getRawRelatedItems",
            "in_reply_to",
            "meta_type",
            "object_provides",
            "portal_type",
            "SearchableText",
            "sync_uid",
        ]
        return ignored

    def get_indexes(self):
        # Base set of indexes
        indexes = {
            "created": translate(_("Created on"), context=self.request),
            "Creator": translate(_("Creator"), context=self.request),
            "effective": translate(_("Publication date"), context=self.request),  # noqa
            "end": translate(_("End Date"), context=self.request),
            "expires": translate(_("Expiration date"), context=self.request),
            "id": translate(_("ID"), context=self.request),
            "is_folderish": translate(_("Folder"), context=self.request),
            "modified": translate(_("Last modified"), context=self.request),  # noqa
            "review_state": translate(_("Review state"), context=self.request),
            "sortable_title": translate(_("Title"), context=self.request),
            "start": translate(_("Start Date"), context=self.request),
            "Subject": translate(_("Tags"), context=self.request),
            "total_comments": translate(
                _("Total comments"), context=self.request
            ),  # noqa
            "Type": translate(_("Type"), context=self.request),
        }
        # Filter out ignored
        indexes = {k: v for k, v in indexes.items() if k not in self.ignored_indexes}
        # Add in extra metadata indexes
        catalog = getToolByName(self.context, "portal_catalog")
        cat_indexes = [idx for idx in catalog.indexes()]
        for index in cat_indexes:
            if index not in indexes and index not in self.ignored_indexes:
                indexes[index] = translate(_(index), context=self.request)
        return indexes

    def get_options(self):
        site = get_top_site_from_url(self.context, self.request)
        base_url = site.absolute_url()
        base_vocabulary = "%s/@@getVocabulary?name=" % base_url
        site_path = site.getPhysicalPath()
        context_path = self.context.getPhysicalPath()
        columns = self.get_columns()
        options = {
            "vocabularyUrl": "%splone.app.vocabularies.Catalog" % (base_vocabulary),
            "urlStructure": {"base": base_url, "appended": "/folder_contents"},
            "moveUrl": "%s{path}/fc-itemOrder" % base_url,
            "indexOptionsUrl": "%s/@@qsOptions" % base_url,
            "contextInfoUrl": "%s{path}/@@fc-contextInfo" % base_url,
            "setDefaultPageUrl": "%s{path}/@@fc-setDefaultPage" % base_url,
            "defaultPageTypes": self.default_page_types(),
            "searchParam": "Title",
            "availableColumns": columns,
            "attributes": [
                "Title",
                "path",
                "getURL",
                "getIcon",
                "getMimeIcon",
                "portal_type",
            ]
            + list(columns.keys()),  # noqa
            "buttons": self.get_actions(),
            "rearrange": {
                "properties": self.get_indexes(),
                "url": "%s{path}/@@fc-rearrange" % base_url,
            },
            "basePath": "/" + "/".join(context_path[len(site_path) :]),
            "upload": {
                "relativePath": "@@fileUpload",
                "baseUrl": base_url,
                "initialFolder": IUUID(self.context, None),
                "useTus": TUS_ENABLED,
            },
            "thumb_scale": self.get_thumb_scale(),
        }
        return options

    def __call__(self):
        self.options = json_dumps(self.get_options())
        return super().__call__()


class ContextInfo(BrowserView):

    attributes = [
        "CreationDate",
        "Creator",
        "Description",
        "EffectiveDate",
        "end",
        "exclude_from_nav",
        "getObjSize",
        "getURL",
        "id",
        "is_folderish",
        "last_comment_date",
        "location",
        "ModificationDate",
        "path",
        "portal_type",
        "review_state",
        "start",
        "Subject",
        "Title",
        "total_comments",
        "Type",
        "UID",
    ]

    def __call__(self):
        factories_menu = getUtility(
            IBrowserMenu, name="plone_contentmenu_factory", context=self.context
        ).getMenuItems(self.context, self.request)
        factories = []
        for item in factories_menu:
            if item.get("title") == "folder_add_settings":
                continue
            title = item.get("title", "")
            factories.append(
                {
                    "id": item.get("id"),
                    "title": title
                    and translate(title, context=self.request)
                    or "",  # noqa
                    "action": item.get("action"),
                }
            )

        context = aq_inner(self.context)
        transform = SafeHTML()
        crumbs = []
        top_site = get_top_site_from_url(self.context, self.request)
        while not context == top_site:
            crumbs.append(
                {
                    "id": context.getId(),
                    "title": transform.scrub_html(
                        utils.pretty_title_or_id(context, context)
                    ),
                }
            )
            context = utils.parent(context)

        catalog = getToolByName(self.context, "portal_catalog")
        try:
            brains = catalog(UID=IUUID(self.context), show_inactive=True)
        except TypeError:
            brains = []
        item = None
        if len(brains) > 0:
            obj = brains[0]
            # context here should be site root
            base_path = "/".join(context.getPhysicalPath())
            item = {}
            for attr in self.attributes:
                key = attr
                if key == "path":
                    attr = "getPath"
                val = getattr(obj, attr, None)
                if callable(val):
                    val = val()
                if key == "path":
                    val = val[len(base_path) :]
                if isinstance(val, (bytes, str)):
                    val = transform.scrub_html(val)
                item[key] = val

        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )
        return json_dumps(
            {
                "addButtons": factories,
                "defaultPage": self.context.getDefaultPage(),
                "breadcrumbs": [c for c in reversed(crumbs)],
                "object": item,
            }
        )



import transaction
from plone.protect import CheckAuthenticator, PostOnly
from Products.CMFCore.utils import getToolByName
from Products.CMFPlone import PloneMessageFactory as _
from Products.CMFPlone.utils import transaction_note
from ZODB.POSException import ConflictError
from zope.component import getMultiAdapter
from zope.publisher.browser import BrowserView


class FolderPublishView(BrowserView):
    """Publish objects from a folder.

    Originally: Products/CMFPlone/skins/plone_scripts/folder_publish.cpy
    Called by content_status_history, in plone.app.content.
    """

    def __call__(
        self,
        workflow_action=None,
        paths=None,
        comment="No comment",
        expiration_date=None,
        effective_date=None,
        include_children=False,
    ):
        # Use plone.protect.
        PostOnly(self.request)
        CheckAuthenticator(self.request)

        plone_utils = getToolByName(self.context, "plone_utils")
        if workflow_action is None:
            plone_utils.addPortalMessage(
                _("You must select a publishing action."), "error"
            )
            return self.redirect()
        if not paths:
            plone_utils.addPortalMessage(
                _("You must select content to change."), "error"
            )
            return self.redirect()

        self.transition_objects_by_paths(
            workflow_action,
            paths,
            comment,
            expiration_date,
            effective_date,
            include_children,
        )

        transaction_note(str(paths) + " transitioned " + workflow_action)
        plone_utils.addPortalMessage(_("Item state changed."))
        return self.redirect()

    def transition_objects_by_paths(
        self,
        workflow_action,
        paths,
        comment="",
        expiration_date=None,
        effective_date=None,
        include_children=False,
        handle_errors=True,
    ):
        """Originally this was in plone_utils.transitionObjectsByPaths.

        This was deprecated since 2015, so we copied it here.
        """
        failure = {}
        # use the portal for traversal in case we have relative paths
        portal = getToolByName(self, "portal_url").getPortalObject()
        traverse = portal.restrictedTraverse
        for path in paths:
            sp = transaction.savepoint(optimistic=True)
            try:
                obj = traverse(path, None)
                if obj is not None:
                    view = getMultiAdapter(
                        (obj, self.request), name="content_status_modify"
                    )
                    view(
                        workflow_action,
                        comment,
                        effective_date=effective_date,
                        expiration_date=expiration_date,
                    )
            except (ConflictError, KeyboardInterrupt):
                raise
            except Exception as e:
                # skip this object but continue with sub-objects.
                sp.rollback()
                failure[path] = e
            if getattr(obj, "isPrincipiaFolderish", None) and include_children:
                subobject_paths = [f"{path}/{id}" for id in obj]
                self.transition_objects_by_paths(
                    workflow_action,
                    subobject_paths,
                    comment,
                    expiration_date,
                    effective_date,
                    include_children,
                )
        return failure

    def redirect(self):
        target = self.request.get("orig_template", "")
        if target and not getToolByName(self.context, "portal_url").isURLInPortal(
            target
        ):
            target = ""
        if not target:
            target = self.context.absolute_url()
        self.request.response.redirect(target)



import json

from plone.app.querystring.interfaces import IQuerystringRegistryReader
from plone.registry.interfaces import IRegistry
from Products.Five import BrowserView
from zope.component import getUtility


class QueryStringIndexOptions(BrowserView):
    def __call__(self):
        registry = getUtility(IRegistry)
        config = IQuerystringRegistryReader(registry)()
        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )
        return json.dumps(config)



import unittest

from plone.app.testing import TEST_USER_ID, TEST_USER_NAME, login, setRoles
from Products.CMFPlone.utils import isExpired
from zope.component import getMultiAdapter

from plone.app.content.testing import PLONE_APP_CONTENT_DX_INTEGRATION_TESTING


class TestContentStatusModify(unittest.TestCase):
    """The the content_status_modify view.

    Until and including Plone 5.2, this was a skin script in Products.CMFPlone.
    Tests adapted from CMFPlone/tests/testContentPublishing.py.
    """

    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.workflow = self.portal.portal_workflow
        # Make sure we can create and publish directly.
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        # Prepare content.
        self.portal.invokeFactory("Folder", id="folder")
        self.folder = self.portal.folder
        self.folder.invokeFactory("Document", id="d1", title="Doc 1")

    def setup_authenticator(self):
        from plone.protect.authenticator import createToken

        self.request.form["_authenticator"] = createToken()

    def get_content_status_modify_view(self, obj):
        self.setup_authenticator()
        view = getMultiAdapter((obj, self.request), name="content_status_modify")
        return view

    # Test the recursive behaviour of content_status_modify:

    def testPublishingNonDefaultPageLeavesFolderAlone(self):
        view = self.get_content_status_modify_view(self.folder.d1)
        view("publish")
        self.assertEqual(
            self.workflow.getInfoFor(self.folder, "review_state"), "private"
        )
        self.assertEqual(
            self.workflow.getInfoFor(self.folder.d1, "review_state"), "published"
        )

    def testPublishingDefaultPagePublishesFolder(self):
        self.folder.setDefaultPage("d1")
        view = self.get_content_status_modify_view(self.folder.d1)
        view("publish")
        self.assertEqual(
            self.workflow.getInfoFor(self.folder, "review_state"), "published"
        )
        self.assertEqual(
            self.workflow.getInfoFor(self.folder.d1, "review_state"), "published"
        )

    def testPublishingDefaultPageWhenFolderCannotBePublished(self):
        self.folder.setDefaultPage("d1")
        # make parent be published already when publishing its default document
        # results in an attempt to do it again
        view = self.get_content_status_modify_view(self.folder)
        view("publish")
        self.assertEqual(
            self.workflow.getInfoFor(self.folder, "review_state"), "published"
        )
        view = self.get_content_status_modify_view(self.folder.d1)
        view("publish")
        self.assertEqual(
            self.workflow.getInfoFor(self.folder, "review_state"), "published"
        )
        self.assertEqual(
            self.workflow.getInfoFor(self.folder.d1, "review_state"), "published"
        )

    # test setting effective/expiration date and isExpired method

    def testIsExpiredWithExplicitExpiredContent(self):
        view = self.get_content_status_modify_view(self.folder.d1)
        view(
            workflow_action="publish",
            effective_date="1/1/2001",
            expiration_date="1/2/2001",
        )
        self.assertTrue(isExpired(self.folder.d1))

    def testIsExpiredWithExplicitNonExpiredContent(self):
        view = self.get_content_status_modify_view(self.folder.d1)
        view(workflow_action="publish")
        self.assertFalse(isExpired(self.folder.d1))



import json
import unittest
from datetime import datetime, timedelta
from unittest import mock

import transaction
from plone.app.testing import (
    SITE_OWNER_NAME,
    SITE_OWNER_PASSWORD,
    TEST_USER_ID,
    TEST_USER_NAME,
    login,
    setRoles,
)
from plone.dexterity.fti import DexterityFTI
from plone.protect.authenticator import createToken
from plone.registry.interfaces import IRegistry
from plone.testing.zope import Browser
from plone.uuid.interfaces import IUUID
from zope.component import getMultiAdapter, getUtility

from plone.app.content.testing import (
    PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING,
    PLONE_APP_CONTENT_DX_INTEGRATION_TESTING,
)


class ContentsCopyTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        # TYPE 1
        type1_fti = DexterityFTI("type1")
        type1_fti.klass = "plone.dexterity.content.Container"
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ["type1"]
        type1_fti.behaviors = (
            "Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes",  # noqa
            "plone.app.dexterity.behaviors.metadata.IBasic",
        )
        self.portal.portal_types._setObject("type1", type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_keep_selection_order(self):
        """Keep the order of items the same as they were selected."""
        self.portal.invokeFactory("type1", id="f1", title="Folder 1")
        f1 = self.portal.f1
        f1.invokeFactory("type1", id="it1", title="Item 1")
        f1.invokeFactory("type1", id="it2", title="Item 2")
        f1.invokeFactory("type1", id="it3", title="Item 3")

        def _test_order(sel):
            self.request.form["selection"] = json.dumps([IUUID(f1[id_]) for id_ in sel])
            view = f1.restrictedTraverse("@@fc-copy")
            view()
            self.assertEqual([ob.id for ob in view.oblist], sel)

        _test_order(["it1", "it2", "it3"])
        _test_order(["it3", "it1", "it2"])


class ContentsDeleteTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        # TYPE 1
        type1_fti = DexterityFTI("type1")
        type1_fti.klass = "plone.dexterity.content.Container"
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ["type1"]
        type1_fti.behaviors = (
            "Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes",  # noqa
            "plone.app.dexterity.behaviors.metadata.IBasic",
        )
        self.portal.portal_types._setObject("type1", type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_delete_success_with_private_anchestor(self):
        """Delete content item from a folder with private anchestor"""
        # Create test content /it1/it2/it3
        self.portal.invokeFactory("type1", id="it1", title="Item 1")
        self.portal.it1.invokeFactory("type1", id="it2", title="Item 2")
        self.portal.it1.it2.invokeFactory("type1", id="it3", title="Item 3")
        self.assertEqual(len(self.portal.it1.it2.contentIds()), 1)

        # Block user access to it1m but leave access to its children
        self.portal.it1.__ac_local_roles_block__ = True
        del self.portal.it1.__ac_local_roles__[TEST_USER_ID]
        self.portal.it1.reindexObjectSecurity()
        self.portal.it1.it2.reindexObjectSecurity()

        # Remove test user global roles (leaving only local owner roles on it2)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute delete request
        selection = [self.portal.it1.it2.it3.UID()]
        self.request.form["folder"] = "/it1/it2"
        self.request.form["selection"] = json.dumps(selection)
        res = self.portal.it1.it2.restrictedTraverse("@@fc-delete")()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res["status"], "success")
        self.assertEqual(len(self.portal.it1.it2.contentIds()), 0)

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_delete_success_on_inactive_content(self):
        """Delete an expired content item from a folder."""
        # Create content
        self.portal.invokeFactory("type1", id="it1", title="Item 1")
        self.portal.it1.invokeFactory("type1", id="it2", title="Item 2")

        # Expire it2
        exp = datetime.now() - timedelta(days=10)
        self.portal.it1.it2.expiration_date = exp
        self.portal.it1.it2.reindexObject()

        # Remove test user global roles (leaving only local owner roles on it1
        # and below)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute delete request
        selection = [self.portal.it1.it2.UID()]
        self.request.form["folder"] = "/it1"
        self.request.form["selection"] = json.dumps(selection)
        res = self.portal.it1.restrictedTraverse("@@fc-delete")()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res["status"], "success")
        self.assertEqual(len(self.portal.it1.contentIds()), 0)


class ContentsPasteTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        # TYPE 1
        type1_fti = DexterityFTI("type1")
        type1_fti.klass = "plone.dexterity.content.Container"
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ["type1"]
        type1_fti.behaviors = (
            "Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes",  # noqa
            "plone.app.dexterity.behaviors.metadata.IBasic",
        )
        self.portal.portal_types._setObject("type1", type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.portal.invokeFactory("type1", id="it1", title="Item 1")

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_paste_success(self):
        """Copy content item and paste in portal root."""
        # # setup copying via @@fc-copy
        # from plone.uuid.interfaces import IUUID
        # self.request['selection'] = [IUUID(self.portal.it1)]
        # self.portal.restrictedTraverse('@@fc-copy')()

        self.request["__cp"] = self.portal.manage_copyObjects(["it1"])
        self.request.form["folder"] = "/"
        res = self.portal.restrictedTraverse("@@fc-paste")()

        res = json.loads(res)
        self.assertEqual(res["status"], "success")
        self.assertEqual(len(self.portal.contentIds()), 2)

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_paste_success_paste_in_itself(self):
        """Copy content item and paste in itself. Because we can."""
        self.request["__cp"] = self.portal.manage_copyObjects(["it1"])
        self.request.form["folder"] = "/it1"
        res = self.portal.it1.restrictedTraverse("@@fc-paste")()

        res = json.loads(res)
        self.assertEqual(res["status"], "success")
        self.assertEqual(len(self.portal.it1.contentIds()), 1)

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_paste_fail_constraint(self):
        """Fail pasting content item in itself when folder constraints don't
        allow to.
        """
        self.type1_fti.allowed_content_types = []  # set folder constraints
        self.request["__cp"] = self.portal.manage_copyObjects(["it1"])
        self.request.form["folder"] = "/it1"
        res = self.portal.it1.restrictedTraverse("@@fc-paste")()

        res = json.loads(res)
        self.assertEqual(res["status"], "warning")
        self.assertEqual(len(self.portal.it1.contentIds()), 0)

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_paste_success_with_private_anchestor(self):
        """Copy content item and paste into a folder with private anchestor"""
        # Create test content /it2/it3
        self.portal.invokeFactory("type1", id="it2", title="Item 2")
        self.portal.it2.invokeFactory("type1", id="it3", title="Item 3")
        self.assertEqual(len(self.portal.it2.it3.contentIds()), 0)

        # Block user access to it2, but leave access to its children
        self.portal.it2.__ac_local_roles_block__ = True
        del self.portal.it2.__ac_local_roles__[TEST_USER_ID]
        self.portal.it2.reindexObjectSecurity()
        self.portal.it2.it3.reindexObjectSecurity()

        # Remove test user global roles (leaving only local owner roles on it2)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute paste
        self.request["__cp"] = self.portal.manage_copyObjects(["it1"])
        self.request.form["folder"] = "/it2/it3"
        res = self.portal.it2.it3.restrictedTraverse("@@fc-paste")()

        # Check for successful paste
        res = json.loads(res)
        self.assertEqual(res["status"], "success")
        self.assertEqual(len(self.portal.it2.it3.contentIds()), 1)


class ContentsRenameTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        # TYPE 1
        type1_fti = DexterityFTI("type1")
        type1_fti.klass = "plone.dexterity.content.Container"
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = ["type1"]
        type1_fti.behaviors = (
            "Products.CMFPlone.interfaces.constrains.ISelectableConstrainTypes",  # noqa
            "plone.app.dexterity.behaviors.metadata.IBasic",
        )
        self.portal.portal_types._setObject("type1", type1_fti)
        self.type1_fti = type1_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_rename_success_with_private_anchestor(self):
        """Rename content item from a folder with private anchestor"""
        # Create test content /it1/it2/it3
        self.portal.invokeFactory("type1", id="it1", title="Item 1")
        self.portal.it1.invokeFactory("type1", id="it2", title="Item 2")
        self.portal.it1.it2.invokeFactory("type1", id="it3", title="Item 3")
        self.assertEqual(len(self.portal.it1.it2.contentIds()), 1)

        # Block user access to it1m but leave access to its children
        self.portal.it1.__ac_local_roles_block__ = True
        del self.portal.it1.__ac_local_roles__[TEST_USER_ID]
        self.portal.it1.reindexObjectSecurity()
        self.portal.it1.it2.reindexObjectSecurity()

        # Remove test user global roles (leaving only local owner roles on it2)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute rename request
        self.request.form["UID_1"] = self.portal.it1.it2.it3.UID()
        self.request.form["newid_1"] = "it3bak"
        self.request.form["newtitle_1"] = "Item 3 BAK"
        res = self.portal.it1.it2.restrictedTraverse("@@fc-rename")()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res["status"], "success")
        self.assertEqual(self.portal.it1.it2.it3bak.id, "it3bak")
        self.assertEqual(self.portal.it1.it2.it3bak.title, "Item 3 BAK")

    @mock.patch(
        "plone.app.content.browser.contents.ContentsBaseAction.protect", lambda x: True
    )  # noqa
    def test_rename_success_on_inactive_content(self):
        """Rename an expired content item from a folder."""
        # Create content
        self.portal.invokeFactory("type1", id="it1", title="Item 1")
        self.portal.it1.invokeFactory("type1", id="it2", title="Item 2")

        # Expire it2
        exp = datetime.now() - timedelta(days=10)
        self.portal.it1.it2.expiration_date = exp
        self.portal.it1.it2.reindexObject()

        # Remove test user global roles (leaving only local owner roles on it1
        # and below)
        setRoles(self.portal, TEST_USER_ID, [])

        # Execute rename request
        self.request.form["UID_1"] = self.portal.it1.it2.UID()
        self.request.form["newid_1"] = "it2bak"
        self.request.form["newtitle_1"] = "Item 2 BAK"
        res = self.portal.it1.restrictedTraverse("@@fc-rename")()

        # Check for successful deletion
        res = json.loads(res)
        self.assertEqual(res["status"], "success")
        self.assertEqual(self.portal.it1.it2bak.id, "it2bak")
        self.assertEqual(self.portal.it1.it2bak.title, "Item 2 BAK")


class AllowUploadViewTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]

        # TYPE 1
        type1_fti = DexterityFTI("type1")
        type1_fti.klass = "plone.dexterity.content.Container"
        type1_fti.filter_content_types = True
        type1_fti.allowed_content_types = []
        type1_fti.behaviors = ("plone.app.dexterity.behaviors.metadata.IBasic",)
        self.portal.portal_types._setObject("type1", type1_fti)
        self.type1_fti = type1_fti

        # TYPE 2
        type2_fti = DexterityFTI("type1")
        type2_fti.klass = "plone.dexterity.content.Item"
        type2_fti.filter_content_types = True
        type2_fti.allowed_content_types = []
        type2_fti.behaviors = ("plone.app.dexterity.behaviors.metadata.IBasic",)
        self.portal.portal_types._setObject("type2", type2_fti)
        self.type2_fti = type2_fti

        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.portal.invokeFactory("type1", id="it1", title="Item 1")
        self.portal.invokeFactory("type2", id="it2", title="Item 2")

    def test_allow_upload(self):
        """Test, if file or images are allowed in a container in different FTI
        configurations.
        """

        # Test non-container, none allowed
        allow_upload = self.portal.it2.restrictedTraverse("@@allow_upload")
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload["allowUpload"], False)
        self.assertEqual(allow_upload["allowImages"], False)
        self.assertEqual(allow_upload["allowFiles"], False)

        # Test none allowed
        self.type1_fti.allowed_content_types = []
        allow_upload = self.portal.it1.restrictedTraverse("@@allow_upload")
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload["allowUpload"], False)
        self.assertEqual(allow_upload["allowImages"], False)
        self.assertEqual(allow_upload["allowFiles"], False)

        # Test images allowed
        self.type1_fti.allowed_content_types = ["Image"]
        allow_upload = self.portal.it1.restrictedTraverse("@@allow_upload")
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload["allowUpload"], True)
        self.assertEqual(allow_upload["allowImages"], True)
        self.assertEqual(allow_upload["allowFiles"], False)

        # Test files allowed
        self.type1_fti.allowed_content_types = ["File"]
        allow_upload = self.portal.it1.restrictedTraverse("@@allow_upload")
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload["allowUpload"], True)
        self.assertEqual(allow_upload["allowImages"], False)
        self.assertEqual(allow_upload["allowFiles"], True)

        # Test images and files allowed
        self.type1_fti.allowed_content_types = ["Image", "File"]
        allow_upload = self.portal.it1.restrictedTraverse("@@allow_upload")
        allow_upload = json.loads(allow_upload())

        self.assertEqual(allow_upload["allowUpload"], True)
        self.assertEqual(allow_upload["allowImages"], True)
        self.assertEqual(allow_upload["allowFiles"], True)

        # Test files allowed, path via request variable
        self.type1_fti.allowed_content_types = ["File"]
        # First, test on Portal root to see the difference
        allow_upload = self.portal.restrictedTraverse("@@allow_upload")
        allow_upload = json.loads(allow_upload())
        self.assertEqual(allow_upload["allowUpload"], True)
        self.assertEqual(allow_upload["allowImages"], True)
        self.assertEqual(allow_upload["allowFiles"], True)
        # Then, with path set to sub item
        allow_upload = self.portal.restrictedTraverse("@@allow_upload")
        allow_upload.request.form["path"] = "/plone/it1"
        allow_upload = json.loads(allow_upload())
        self.assertEqual(allow_upload["allowUpload"], True)
        self.assertEqual(allow_upload["allowImages"], False)
        self.assertEqual(allow_upload["allowFiles"], True)


class FCPropertiesTests(unittest.TestCase):
    layer = PLONE_APP_CONTENT_DX_INTEGRATION_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        # Disable plone.protect for these tests
        self.request.environ["REQUEST_METHOD"] = "POST"
        self.request.form["_authenticator"] = createToken()
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        # set available languages
        registry = getUtility(IRegistry)
        registry["plone.available_languages"] = ["en", "de"]

        self.portal.invokeFactory("Folder", "main1")
        self.portal.main1.invokeFactory("Folder", "sub1")
        self.portal.main1.sub1.invokeFactory("Folder", "subsub1")
        self.portal.main1.invokeFactory("Document", "sub2")
        self.portal.invokeFactory("Document", "main2")

        self.setup_initial()

    def setup_initial(self):
        # Initial Settings
        self.portal.main1.exclude_from_nav = True
        self.portal.main1.sub1.exclude_from_nav = True
        self.portal.main1.sub1.subsub1.exclude_from_nav = True
        self.portal.main1.sub2.exclude_from_nav = True
        self.portal.main2.exclude_from_nav = True

        self.portal.main1.language = "en"
        self.portal.main1.sub1.language = "en"
        self.portal.main1.sub1.subsub1.language = "en"
        self.portal.main1.sub2.language = "en"
        self.portal.main2.language = "en"

    def test_fc_properties__changes__no_recurse(self):
        """Test changing properties without recursion."""
        req = self.request
        req.form["language"] = "de"
        req.form["exclude-from-nav"] = "no"
        req.form["selection"] = '["{}", "{}"]'.format(
            IUUID(self.portal.main1), IUUID(self.portal.main2)
        )

        view = getMultiAdapter((self.portal, req), name="fc-properties")

        # Call the view and execute the actions
        view()

        self.assertEqual(self.portal.main1.language, "de")
        self.assertEqual(self.portal.main2.language, "de")
        self.assertEqual(self.portal.main1.sub1.language, "en")
        self.assertEqual(self.portal.main1.sub1.subsub1.language, "en")
        self.assertEqual(self.portal.main1.sub2.language, "en")

        self.assertEqual(self.portal.main1.exclude_from_nav, False)
        self.assertEqual(self.portal.main2.exclude_from_nav, False)
        self.assertEqual(self.portal.main1.sub1.exclude_from_nav, True)
        self.assertEqual(self.portal.main1.sub1.subsub1.exclude_from_nav, True)
        self.assertEqual(self.portal.main1.sub2.exclude_from_nav, True)

    def test_fc_properties__changes__with_recurse(self):
        """Test changing properties without recursion."""
        req = self.request
        req.form["language"] = "de"
        req.form["exclude-from-nav"] = "no"
        req.form["recurse"] = "yes"
        req.form["selection"] = '["{}", "{}"]'.format(
            IUUID(self.portal.main1), IUUID(self.portal.main2)
        )

        view = getMultiAdapter((self.portal, req), name="fc-properties")

        # Call the view and execute the actions
        view()

        self.assertEqual(self.portal.main1.language, "de")
        self.assertEqual(self.portal.main2.language, "de")
        self.assertEqual(self.portal.main1.sub1.language, "de")
        self.assertEqual(self.portal.main1.sub1.subsub1.language, "de")
        self.assertEqual(self.portal.main1.sub2.language, "de")

        self.assertEqual(self.portal.main1.exclude_from_nav, False)
        self.assertEqual(self.portal.main2.exclude_from_nav, False)
        self.assertEqual(self.portal.main1.sub1.exclude_from_nav, False)
        self.assertEqual(self.portal.main1.sub1.subsub1.exclude_from_nav, False)  # noqa
        self.assertEqual(self.portal.main1.sub2.exclude_from_nav, False)


# We want to avoid hackers getting script tags inserted.
# But for example an ampersand is okay as long as it is escaped,
# although it should not be doubly escaped, because that looks wrong.
NORMAL_TEXT = "Smith & Jones"
ESCAPED_TEXT = "Smith &amp; Jones"
DOUBLY_ESCAPED_TEXT = "Smith &amp;amp; Jones"
# For script tags, safest is to filter them using the safe html filter.
HACKED = 'The <script>alert("hacker")</script> was here.'


class TestSafeHtmlInFolderContents(unittest.TestCase):
    """Test that the title in the folder contents is safe.

    From PloneHotfix20200121, see
    https://plone.org/security/hotfix/20200121/xss-in-the-title-field-on-plone-5-0-and-higher

    Same for other fields, from PloneHotfix20210518, see
    https://plone.org/security/hotfix/20210518/stored-xss-in-folder-contents
    """

    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        login(self.portal, TEST_USER_NAME)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def get_browser(self):
        browser = Browser(self.layer["app"])
        browser.handleErrors = False
        browser.addHeader(
            "Authorization",
            f"Basic {SITE_OWNER_NAME}:{SITE_OWNER_PASSWORD}",
        )
        return browser

    def test_ampersand(self):
        self.portal.invokeFactory(
            "Folder",
            id="folder1",
            title=NORMAL_TEXT,
            description=NORMAL_TEXT,
            creators=(NORMAL_TEXT,),
            contributors=(NORMAL_TEXT,),
        )
        folder1 = self.portal.folder1
        self.assertEqual(folder1.Title(), NORMAL_TEXT)
        self.assertEqual(folder1.Description(), NORMAL_TEXT)
        folder1.invokeFactory(
            "Document",
            id="page1",
            title=NORMAL_TEXT,
            description=NORMAL_TEXT,
            creators=(NORMAL_TEXT,),
            contributors=(NORMAL_TEXT,),
        )
        page1 = folder1.page1
        self.assertEqual(page1.Title(), NORMAL_TEXT)
        self.assertEqual(page1.Description(), NORMAL_TEXT)
        transaction.commit()

        # Check the output.
        browser = self.get_browser()
        browser.open(folder1.absolute_url())
        self.assert_only_escaped_text(browser)
        browser.open(page1.absolute_url())
        self.assert_only_escaped_text(browser)
        browser.open(folder1.absolute_url() + "/folder_contents")
        self.assert_only_escaped_text(browser)

        browser.open(folder1.absolute_url() + "/@@fc-contextInfo")
        self.assert_only_escaped_text(browser)

    def test_xss(self):
        self.portal.invokeFactory(
            "Folder",
            id="folder1",
            title=HACKED,
            description=HACKED,
            creators=(HACKED,),
            contributors=(HACKED,),
        )
        folder1 = self.portal.folder1
        self.assertEqual(folder1.Title(), HACKED)
        # With good old Archetypes the description gets cleaned up to
        # 'The  alert("hacker")  was here.'
        # self.assertEqual(folder1.Description(), HACKED)
        folder1.invokeFactory(
            "Document",
            id="page1",
            title=HACKED,
            description=HACKED,
            creators=(HACKED,),
            contributors=(HACKED,),
        )
        page1 = folder1.page1
        self.assertEqual(page1.Title(), HACKED)
        # self.assertEqual(page1.Description(), HACKED)
        transaction.commit()

        # Check the output.
        browser = self.get_browser()
        browser.open(folder1.absolute_url())
        self.assert_not_in(HACKED, browser.contents)
        browser.open(page1.absolute_url())
        self.assert_not_in(HACKED, browser.contents)
        browser.open(folder1.absolute_url() + "/folder_contents")
        self.assert_not_in(HACKED, browser.contents)

        browser.open(folder1.absolute_url() + "/@@fc-contextInfo")
        self.assert_not_in(HACKED, browser.contents)

    def assert_only_escaped_text(self, browser):
        body = browser.contents
        # The escaped version of the text text should be in the response text.
        self.assertIn(ESCAPED_TEXT, body)
        # The normal version should not.
        self.assert_not_in(NORMAL_TEXT, body)
        # We should avoid escaping twice.
        self.assert_not_in(DOUBLY_ESCAPED_TEXT, body)

    def assert_not_in(self, target, body):
        # This gives a too verbose error message, showing the entire body:
        # self.assertNotIn("x", body)
        # So we roll our own less verbose version.
        if target not in body:
            return
        index = body.index(target)
        start = max(0, index - 50)
        end = min(index + len(target) + 50, len(body))
        assert False, "Text '{}' unexpectedly found in body: ... {} ...".format(
            target, body[start:end]
        )



import doctest
import unittest

from plone.testing import layered

from plone.app.content.testing import (
    PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING,
    optionflags,
)

doctests = ("basecontent.rst",)


def test_suite():
    suite = unittest.TestSuite()
    tests = [
        layered(
            doctest.DocFileSuite(
                test_file,
                package="plone.app.content",
                optionflags=optionflags,
            ),
            layer=PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING,
        )
        for test_file in doctests
    ]
    suite.addTests(tests)
    return suite



from setuptools import find_packages, setup

version = "4.0.0a5"

setup(
    name="plone.app.content",
    version=version,
    description="Content Views for Plone",
    long_description="\n\n".join(
        [
            open("README.rst").read(),
            open("CHANGES.rst").read(),
        ]
    ),
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Framework :: Plone",
        "Framework :: Plone :: 6.0",
        "Framework :: Plone :: Core",
        "Framework :: Zope :: 5",
        "License :: OSI Approved :: GNU General Public License v2 (GPLv2)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
    ],
    keywords="plone content views viewlet",
    author="Plone Foundation",
    author_email="plone-developers@lists.sourceforge.net",
    url="https://pypi.org/project/plone.app.content",
    license="GPL version 2",
    packages=find_packages(),
    namespace_packages=["plone", "plone.app"],
    include_package_data=True,
    zip_safe=False,
    extras_require=dict(
        test=[
            "plone.app.contenttypes",
            "plone.app.testing",
        ]
    ),
    install_requires=[
        "Acquisition",
        "plone.app.widgets",
        "plone.batching",
        "plone.i18n",
        "plone.memoize",
        "plone.protect",
        "Products.CMFCore>=2.2.0dev",
        "Products.CMFDynamicViewFTI",  # required for cmf.ModifyViewTemplate
        "plone.app.vocabularies>4.1.2",
        "setuptools",
        "simplejson",
        "z3c.form",
        "zope.component",
        "zope.container",
        "zope.event",
        "zope.i18n",
        "zope.i18nmessageid",
        "zope.interface",
        "zope.lifecycleevent",
        "zope.publisher",
        "zope.schema",
        "Zope",
    ],
)
